const NSURI_EMPTY = '';
const NSURI_FOO = 'https://example.com/namespace';
const NSURI_HTML = 'http://www.w3.org/1999/xhtml';
const NSURI_MATHML = 'http://www.w3.org/1998/Math/MathML';
const NSURI_RDF = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
const NSURI_SVG = 'http://www.w3.org/2000/svg';
const NSURI_XLINK = 'http://www.w3.org/1999/xlink';
const NSURI_XML = 'http://www.w3.org/XML/1998/namespace';
const NSURI_XMLNS = 'http://www.w3.org/XML/1998/namespace';
const NSURI_XSLT = 'http://www.w3.org/1999/XSL/Transform';
const NSURI_XUL = 'http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul';
const NSURI_ARRAY = [
  NSURI_EMPTY,
  NSURI_FOO,
  NSURI_HTML,
  NSURI_MATHML,
  NSURI_RDF,
  NSURI_SVG,
  NSURI_XLINK,
  NSURI_XML,
  NSURI_XMLNS,
  NSURI_XSLT,
  NSURI_XUL,
];
